// collections.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2002      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

boolean Collection::memberP(Object* object) {
  // Return true iff `object' is a member of the collection `self'.
  { Collection* self = this;

    { Object* i = NULL;
      Iterator* iter000 = ((Iterator*)(self->allocateIterator()));

      while (iter000->nextP()) {
        i = iter000->value;
        if (eqlP(i, object)) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

boolean stellaCollectionP(Object* self) {
  // Return `true' if `self' is a native collection.
  return (isaP(self, SGT_COLLECTIONS_STELLA_COLLECTION));
}

boolean Collection::noDuplicatesP() {
  // Return `true' if the collection `self' forbids duplicate values.
  { Collection* self = this;

    return (FALSE);
  }
}

boolean Set::noDuplicatesP() {
  { Set* self = this;

    return (TRUE);
  }
}

boolean Collection::orderedP() {
  // Return `true' if the collection `self' is ordered.
  { Collection* self = this;

    return (FALSE);
  }
}

boolean Sequence::orderedP() {
  { Sequence* self = this;

    return (TRUE);
  }
}

boolean Set::emptyP() {
  // Return `true' if the set `self' has no members.
  { Set* self = this;

    return (self->theConsList == NIL);
  }
}

boolean Set::nonEmptyP() {
  // Return `true' if the set `self' has at least one member.
  { Set* self = this;

    return (!(self->theConsList == NIL));
  }
}

boolean terminateSetP(Set* self) {
  { Cons* cons = self->theConsList;
    Cons* rest = NULL;

    while (!(cons == NIL)) {
      rest = cons->rest;
      cons->free();
      cons = rest;
    }
    self->theConsList = NIL;
    return (TRUE);
  }
}

boolean Set::memberP(Object* object) {
  // Return `true' iff `object' is a member of the set `self'.
  // Uses an `eql?' test.
  { Set* self = this;

    { Object* i = NULL;
      Cons* iter000 = self->theConsList;

      while (!(iter000 == NIL)) {
        i = iter000->value;
        iter000 = iter000->rest;
        if (eqlP(i, object)) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

int Set::length() {
  { Set* self = this;

    return (self->theConsList->length());
  }
}

void Set::insert(Object* value) {
  // Add `value' to the set `self'.  First checks for duplicate.
  { Set* self = this;

    if (!self->memberP(value)) {
      self->theConsList = cons(value, self->theConsList);
    }
  }
}

AbstractCollection* Set::remove(Object* value) {
  // Remove all entries in `self' that match `value'.
  { Set* self = this;

    self->theConsList = self->theConsList->remove(value);
    return (self);
  }
}

Set* Set::reverse() {
  { Set* self = this;

    self->theConsList = self->theConsList->reverse();
    return (self);
  }
}

Set* Set::copy() {
  // Return a copy of the set `self'.  The conses in the copy are
  // freshly allocated.
  { Set* self = this;

    { Set* copy = ((Set*)(createObject(self->primaryType(), 0)));

      copy->theConsList = copyConsList(self->theConsList);
      return (copy);
    }
  }
}

void Set::clear() {
  // Make `self' an empty set.
  { Set* self = this;

    self->theConsList = NIL;
  }
}

AbstractIterator* Set::allocateIterator() {
  { Set* self = this;

    { SetIterator* iterator = newSetIterator();

      iterator->setIteratorCursor = self->theConsList;
      iterator->setIteratorCollection = self;
      iterator->firstIterationP = TRUE;
      return (iterator);
    }
  }
}

boolean terminateSetIteratorP(SetIterator* self) {
  self->setIteratorCursor = NIL;
  self->setIteratorCollection = NULL;
  return (TRUE);
}

boolean SetIterator::nextP() {
  { SetIterator* self = this;

    if (self->firstIterationP) {
      {
        self->firstIterationP = FALSE;
        self->setIteratorCursor = self->setIteratorCollection->theConsList;
      }
    }
    else {
      self->setIteratorCursor = self->setIteratorCursor->rest;
    }
    self->value = self->setIteratorCursor->value;
    return (!(self->setIteratorCursor == NIL));
  }
}

Object* SetIterator::valueSetter(Object* value) {
  { SetIterator* self = this;

    self->setIteratorCursor->value = value;
    self->value = value;
    return (value);
  }
}

void Vector::printVector(ostream* stream) {
  { Vector* self = this;

    if (self->length() == 0) {
      *(stream) << "|i|[]";
    }
    else {
      { int i = 0;
        int limit = 9;

        *(stream) << "|i|[";
        { Object* element = NULL;
          Vector* vector000 = self;
          int index000 = 0;
          int length000 = vector000->length();

          while (index000 < length000) {
            element = (vector000->theArray)[index000];
            index000 = index000 + 1;
            *(stream) << element;
            i = i + 1;
            if (i > limit) {
              break;
            }
            if (i < self->length()) {
              *(stream) << " ";
            }
          }
        }
        if ((i <= limit) ||
            (i == self->length())) {
          *(stream) << "]";
        }
        else {
          *(stream) << " ...]";
        }
      }
    }
  }
}

boolean Vector::emptyP() {
  // Return `true' if `self' has length 0.
  { Vector* self = this;

    return (self->arraySize == 0);
  }
}

boolean Vector::nonEmptyP() {
  // Return `true' if `self' has length > 0.
  { Vector* self = this;

    return (self->arraySize > 0);
  }
}

Object* Vector::first() {
  { Vector* self = this;

    return ((self->theArray)[0]);
  }
}

Object* Vector::second() {
  { Vector* self = this;

    return ((self->theArray)[1]);
  }
}

Object* Vector::third() {
  { Vector* self = this;

    return ((self->theArray)[2]);
  }
}

Object* Vector::fourth() {
  { Vector* self = this;

    return ((self->theArray)[3]);
  }
}

Object* Vector::fifth() {
  { Vector* self = this;

    return ((self->theArray)[4]);
  }
}

Object* Vector::nth(int position) {
  { Vector* self = this;

    return ((self->theArray)[position]);
  }
}

Object* Vector::last() {
  // Return the last item in the vector `self'.
  { Vector* self = this;

    return ((self->theArray)[(self->arraySize - 1)]);
  }
}

Object* Vector::firstSetter(Object* value) {
  { Vector* self = this;

    return ((self->theArray)[0] = value);
  }
}

Object* Vector::secondSetter(Object* value) {
  { Vector* self = this;

    return ((self->theArray)[1] = value);
  }
}

Object* Vector::thirdSetter(Object* value) {
  { Vector* self = this;

    return ((self->theArray)[2] = value);
  }
}

Object* Vector::fourthSetter(Object* value) {
  { Vector* self = this;

    return ((self->theArray)[3] = value);
  }
}

Object* Vector::fifthSetter(Object* value) {
  { Vector* self = this;

    return ((self->theArray)[4] = value);
  }
}

Object* Vector::nthSetter(Object* value, int position) {
  { Vector* self = this;

    return ((self->theArray)[position] = value);
  }
}

Object* Vector::lastSetter(Object* value) {
  { Vector* self = this;

    return ((self->theArray)[(self->arraySize - 1)] = value);
  }
}

int Vector::length() {
  { Vector* self = this;

    return (self->arraySize);
  }
}

boolean Vector::memberP(Object* object) {
  { Vector* self = this;

    { Object** array = self->theArray;

      { int i = NULL_INTEGER;
        int iter000 = 0;
        int upperBound000 = self->length() - 1;

        while (iter000 <= upperBound000) {
          i = iter000;
          iter000 = iter000 + 1;
          if (eqlP(array[i], object)) {
            return (TRUE);
          }
        }
      }
      return (FALSE);
    }
  }
}

int Vector::position(Object* object, int start) {
  // Return the position of `object' within the vector
  // `self' (counting from zero); or return `null' if `object' does not occur within 
  // `self' (uses an `eql?' test).  If `start' was supplied as non-`null', only 
  // consider the portion starting at `start', however, the returned position 
  // will always be relative to the entire vector.
  { Vector* self = this;

    if (start == NULL_INTEGER) {
      start = 0;
    }
    { int i = NULL_INTEGER;
      int iter000 = start;
      int upperBound000 = self->length() - 1;

      while (iter000 <= upperBound000) {
        i = iter000;
        iter000 = iter000 + 1;
        if (eqlP((self->theArray)[i], object)) {
          return (i);
        }
      }
    }
    return (NULL_INTEGER);
  }
}

void Vector::insertAt(int offset, Object* value) {
  { Vector* self = this;

    if (offset >= self->arraySize) {
      throw *newStellaException("Out-of-bounds array access");
    }
    (self->theArray)[offset] = value;
  }
}

Vector* Vector::copy() {
  // Return a copy of the vector `self'.
  { Vector* self = this;

    { int size = self->arraySize;
      Object** sourcearray = self->theArray;
      Vector* copy = newVector(size);
      Object** copyarray = copy->theArray;

      { int i = NULL_INTEGER;
        int iter000 = 0;
        int upperBound000 = size - 1;

        while (iter000 <= upperBound000) {
          i = iter000;
          iter000 = iter000 + 1;
          copyarray[i] = (sourcearray[i]);
        }
      }
      return (copy);
    }
  }
}

void Vector::clear() {
  { Vector* self = this;

    { int size = self->arraySize;
      Object** array = self->theArray;

      { int i = NULL_INTEGER;
        int iter000 = 0;
        int upperBound000 = size - 1;

        while (iter000 <= upperBound000) {
          i = iter000;
          iter000 = iter000 + 1;
          array[i] = NULL;
        }
      }
    }
  }
}

Vector* vector_(int values, ...) {
  // Return a vector containing `values', in order.
  { Vector* vector = newVector(values);

    { Object* v = NULL;
      int i = NULL_INTEGER;
      va_list iter000;
      int iter000Count = values;
      int iter001 = 0;

      va_start(iter000, values);
      while ((iter000Count--) > 0) {
        v = va_arg(iter000, Object*);
        i = iter001;
        iter001 = iter001 + 1;
        (vector->theArray)[i] = v;
      }
      va_end(iter000);
    }
    return (vector);
  }
}

boolean Vector::objectEqualP(Object* y) {
  // Return TRUE iff the vectors `x' and `y' are structurally
  // equivalent.  Uses `equal?' to test equality of elements.
  { Vector* x = this;

    if (subtypeOfP(safePrimaryType(y), SGT_COLLECTIONS_STELLA_VECTOR)) {
      { Object* y000 = y;
        Vector* y = ((Vector*)(y000));

        if (x->arraySize == y->arraySize) {
          { Object* eltx = NULL;
            Object* elty = NULL;
            Vector* vector000 = x;
            int index000 = 0;
            int length000 = vector000->length();
            Vector* vector001 = y;
            int index001 = 0;
            int length001 = vector001->length();

            while ((index000 < length000) &&
                (index001 < length001)) {
              eltx = (vector000->theArray)[index000];
              index000 = index000 + 1;
              elty = (vector001->theArray)[index001];
              index001 = index001 + 1;
              if (!equalP(eltx, elty)) {
                return (FALSE);
              }
            }
          }
          return (TRUE);
        }
      }
    }
    else {
    }
    return (FALSE);
  }
}

void ExtensibleVector::insertAt(int offset, Object* value) {
  { ExtensibleVector* self = this;

    if (offset >= self->arraySize) {
      { int size = self->arraySize;

        while (offset >= size) {
          size = 2 * size;
        }
        resizeVector(self, size);
      }
    }
    (self->theArray)[offset] = value;
  }
}

void ExtensibleSymbolArray::insertAt(int offset, Object* value) {
  { ExtensibleSymbolArray* self = this;

    if (offset >= self->arraySize) {
      { int size = self->arraySize;

        while (offset >= size) {
          size = 2 * size;
        }
        resizeVector(self, size);
      }
    }
    (self->theArray)[offset] = value;
    if (offset > self->topSymbolOffset) {
      self->topSymbolOffset = offset;
    }
  }
}

void VectorSequence::insert(Object* value) {
  // Append `value' to the END of the sequence `self'.  
  // Resize the array if necessary.
  { VectorSequence* self = this;

    { int oldlength = self->sequenceLength;

      if (oldlength == self->arraySize) {
        resizeVector(self, self->arraySize + self->resizeIncrement_reader());
      }
      (self->theArray)[oldlength] = value;
      self->sequenceLength = oldlength + 1;
    }
  }
}

AbstractCollection* VectorSequence::remove(Object* value) {
  // Remove `value' from the sequence `self', and left shift
  // the values after it to close the gap.
  { VectorSequence* self = this;

    { Object** array = self->theArray;
      int firstshiftoffset = -1;
      int lastshiftoffset = self->sequenceLength - 1;

      { int i = NULL_INTEGER;
        int iter000 = 0;
        int upperBound000 = lastshiftoffset;
        boolean unboundedP000 = upperBound000 == NULL_INTEGER;

        while (unboundedP000 ||
            (iter000 <= upperBound000)) {
          i = iter000;
          iter000 = iter000 + 1;
          if (eqlP(array[i], value)) {
            firstshiftoffset = i + 1;
            break;
          }
        }
      }
      if (firstshiftoffset == -1) {
        return (self);
      }
      if (firstshiftoffset <= lastshiftoffset) {
        { int j = NULL_INTEGER;
          int iter001 = firstshiftoffset;
          int upperBound001 = lastshiftoffset;
          boolean unboundedP001 = upperBound001 == NULL_INTEGER;

          while (unboundedP001 ||
              (iter001 <= upperBound001)) {
            j = iter001;
            iter001 = iter001 + 1;
            array[(j - 1)] = (array[j]);
          }
        }
      }
      array[lastshiftoffset] = NULL;
      self->sequenceLength = lastshiftoffset;
      return (self);
    }
  }
}

int VectorSequence::length() {
  { VectorSequence* self = this;

    return (self->sequenceLength);
  }
}

AbstractIterator* Vector::allocateIterator() {
  { Vector* self = this;

    { AllPurposeIterator* iterator = newAllPurposeIterator();

      iterator->iteratorObject = self;
      iterator->iteratorInteger = 0;
      iterator->iteratorSecondInteger = self->arraySize;
      iterator->iteratorNextCode = ((cpp_function_code)(&vectorNextP));
      return (iterator);
    }
  }
}

boolean vectorNextP(AllPurposeIterator* self) {
  { int cursor = self->iteratorInteger;

    if (cursor < self->iteratorSecondInteger) {
      self->value = (((Vector*)(self->iteratorObject))->theArray)[cursor];
      self->iteratorInteger = cursor + 1;
      return (TRUE);
    }
    return (FALSE);
  }
}

Iterator* Vector::butLast() {
  // Generate all but the last element of the vector `self'.
  { Vector* self = this;

    { AllPurposeIterator* iterator = newAllPurposeIterator();

      iterator->iteratorObject = self;
      iterator->iteratorInteger = 0;
      iterator->iteratorSecondInteger = self->arraySize - 1;
      iterator->iteratorNextCode = ((cpp_function_code)(&vectorNextP));
      return (iterator);
    }
  }
}

boolean oUSE_STELLA_HASH_TABLESpo = FALSE;

StellaHashTable* newStellaHashTable() {
  { StellaHashTable* self = NULL;

    self = new StellaHashTable();
    self->theStellaHashTable = NULL;
    self->theHashTable = NULL;
    self->freeElements = NULL_INTEGER;
    self->initialSize = 50;
    self->size = NULL_INTEGER;
    self->theTable = NULL;
    self->initializeHashTable();
    return (self);
  }
}

Surrogate* StellaHashTable::primaryType() {
  { StellaHashTable* self = this;

    return (SGT_COLLECTIONS_STELLA_STELLA_HASH_TABLE);
  }
}

Object* accessStellaHashTableSlotValue(StellaHashTable* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_COLLECTIONS_STELLA_SIZE) {
    if (setvalueP) {
      self->size = ((IntegerWrapper*)(value))->wrapperValue;
    }
    else {
      value = wrapInteger(self->size);
    }
  }
  else if (slotname == SYM_COLLECTIONS_STELLA_INITIAL_SIZE) {
    if (setvalueP) {
      self->initialSize = ((IntegerWrapper*)(value))->wrapperValue;
    }
    else {
      value = wrapInteger(self->initialSize);
    }
  }
  else if (slotname == SYM_COLLECTIONS_STELLA_FREE_ELEMENTS) {
    if (setvalueP) {
      self->freeElements = ((IntegerWrapper*)(value))->wrapperValue;
    }
    else {
      value = wrapInteger(self->freeElements);
    }
  }
  else {
    { OutputStringStream* stream000 = newOutputStringStream();

      *(stream000->nativeStream) << "`" << slotname << "'" << " is not a valid case option";
      throw *newStellaException(stream000->theStringReader());
    }
  }
  return (value);
}

double oSTELLA_HASH_TABLE_AVG_BUCKET_LENGTHo = 1.0;

void initializeStellaHashTable(StellaHashTable* self) {
  { int size = pickHashTableSizePrime(floor_(self->initialSize / oSTELLA_HASH_TABLE_AVG_BUCKET_LENGTHo));
    KvCons** table = new (GC)(KvCons*[size]);

    { int i = NULL_INTEGER;
      int iter000 = 0;
      int upperBound000 = size - 1;

      while (iter000 <= upperBound000) {
        i = iter000;
        iter000 = iter000 + 1;
        table[i] = NULL;
      }
    }
    self->theTable = table;
    self->size = size;
    self->freeElements = floor_(size * oSTELLA_HASH_TABLE_AVG_BUCKET_LENGTHo);
  }
}

StellaHashTable* makeStellaHashTable(int initialsize) {
  { StellaHashTable* table = new StellaHashTable();

    table->theTable = NULL;
    table->size = NULL_INTEGER;
    table->initialSize = initialsize;
    table->freeElements = NULL_INTEGER;
    return (table);
  }
}

void rehashStellaHashTable(StellaHashTable* self, int newsize) {
  if (self->theTable == NULL) {
    initializeStellaHashTable(self);
    return;
  }
  { int size = self->size;
    KvCons** table = self->theTable;
    KvCons** newtable = new (GC)(KvCons*[newsize]);
    int newbucketindex = 0;
    KvCons* newbucket = NULL;
    KvCons* cursor = NULL;
    KvCons* current = NULL;

    { int i = NULL_INTEGER;
      int iter000 = 0;
      int upperBound000 = newsize - 1;

      while (iter000 <= upperBound000) {
        i = iter000;
        iter000 = iter000 + 1;
        newtable[i] = NULL;
      }
    }
    { int i = NULL_INTEGER;
      int iter001 = 0;
      int upperBound001 = size - 1;

      while (iter001 <= upperBound001) {
        i = iter001;
        iter001 = iter001 + 1;
        cursor = table[i];
        while (((boolean)(cursor))) {
          newbucketindex = ((cursor->key->hashCode()) % newsize);
          newbucket = newtable[newbucketindex];
          current = cursor;
          cursor = cursor->rest;
          if (((boolean)(newbucket))) {
            current->rest = newbucket->rest;
            newbucket->rest = current;
          }
          else {
            newtable[newbucketindex] = current;
            current->rest = NULL;
          }
        }
      }
    }
    self->theTable = newtable;
    self->size = newsize;
    self->freeElements = max_(self->freeElements + floor_((newsize - size) * oSTELLA_HASH_TABLE_AVG_BUCKET_LENGTHo), 0);
  }
}

void stellaHashTableInsertAt(StellaHashTable* self, Object* key, Object* value) {
  { KvCons** table = self->theTable;
    int free = self->freeElements;
    int bucketindex = 0;
    KvCons* bucket = NULL;

    if (table == NULL) {
      initializeStellaHashTable(self);
      table = self->theTable;
      free = self->freeElements;
    }
    if (free == 0) {
      rehashStellaHashTable(self, pickHashTableSizePrime(self->size + 1));
      table = self->theTable;
      free = self->freeElements;
    }
    bucketindex = ((key->hashCode()) % (self->size));
    bucket = table[bucketindex];
    if (!((boolean)(bucket))) {
      { KvCons* self000 = newKvCons();

        self000->key = key;
        self000->value = value;
        table[bucketindex] = self000;
      }
      self->freeElements = free - 1;
    }
    else {
      { KvCons* cursor = bucket;

        while (((boolean)(cursor)) &&
            (!eqlP(cursor->key, key))) {
          cursor = cursor->rest;
        }
        if (((boolean)(cursor))) {
          cursor->value = value;
        }
        else {
          { KvCons* self001 = newKvCons();

            self001->key = key;
            self001->value = value;
            self001->rest = bucket->rest;
            bucket->rest = self001;
          }
          self->freeElements = free - 1;
        }
      }
    }
  }
}

void stellaHashTableRemoveAt(StellaHashTable* self, Object* key) {
  { KvCons** table = self->theTable;
    int bucketindex = 0;
    KvCons* bucket = NULL;

    if (table == NULL) {
      return;
    }
    bucketindex = ((key->hashCode()) % (self->size));
    bucket = table[bucketindex];
    if (!((boolean)(bucket))) {
      return;
    }
    else if (eqlP(bucket->key, key)) {
      table[bucketindex] = bucket->rest;
      self->freeElements = self->freeElements + 1;
      return;
    }
    else {
      { KvCons* trailer = bucket;

        bucket = bucket->rest;
        while (((boolean)(bucket))) {
          if (eqlP(bucket->key, key)) {
            trailer->rest = bucket->rest;
            self->freeElements = self->freeElements + 1;
            return;
          }
          else {
            trailer = bucket;
            bucket = bucket->rest;
          }
        }
      }
    }
  }
}

Object* stellaHashTableLookup(StellaHashTable* self, Object* key) {
  { KvCons** table = self->theTable;
    KvCons* bucket = NULL;

    if (table == NULL) {
      return (NULL);
    }
    bucket = table[((key->hashCode()) % (self->size))];
    while (((boolean)(bucket))) {
      if (eqlP(bucket->key, key)) {
        return (bucket->value);
      }
      else {
        bucket = bucket->rest;
      }
    }
    return (NULL);
  }
}

Object* stellaStringHashTableLookup(StellaHashTable* self, char* key) {
  { KvCons** table = self->theTable;
    KvCons* bucket = NULL;

    if (table == NULL) {
      return (NULL);
    }
    bucket = table[((hashString(key, 0)) % (self->size))];
    while (((boolean)(bucket))) {
      if (stringEqlP(((StringWrapper*)(bucket->key))->wrapperValue, key)) {
        return (bucket->value);
      }
      else {
        bucket = bucket->rest;
      }
    }
    return (NULL);
  }
}

void helpStartupCollections1() {
  {
    SGT_COLLECTIONS_STELLA_COLLECTION = ((Surrogate*)(internRigidSymbolWrtModule("COLLECTION", NULL, 1)));
    SYM_COLLECTIONS_STELLA_VECTOR = ((Symbol*)(internRigidSymbolWrtModule("VECTOR", NULL, 0)));
    KWD_COLLECTIONS_CPP = ((Keyword*)(internRigidSymbolWrtModule("CPP", NULL, 2)));
    KWD_COLLECTIONS_FUNCTION = ((Keyword*)(internRigidSymbolWrtModule("FUNCTION", NULL, 2)));
    SGT_COLLECTIONS_STELLA_VECTOR = ((Surrogate*)(internRigidSymbolWrtModule("VECTOR", NULL, 1)));
    SGT_COLLECTIONS_STELLA_STELLA_HASH_TABLE = ((Surrogate*)(internRigidSymbolWrtModule("STELLA-HASH-TABLE", NULL, 1)));
    SYM_COLLECTIONS_STELLA_SIZE = ((Symbol*)(internRigidSymbolWrtModule("SIZE", NULL, 0)));
    SYM_COLLECTIONS_STELLA_INITIAL_SIZE = ((Symbol*)(internRigidSymbolWrtModule("INITIAL-SIZE", NULL, 0)));
    SYM_COLLECTIONS_STELLA_FREE_ELEMENTS = ((Symbol*)(internRigidSymbolWrtModule("FREE-ELEMENTS", NULL, 0)));
    SYM_COLLECTIONS_STELLA_STARTUP_COLLECTIONS = ((Symbol*)(internRigidSymbolWrtModule("STARTUP-COLLECTIONS", NULL, 0)));
    SYM_COLLECTIONS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(internRigidSymbolWrtModule("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void helpStartupCollections2() {
  {
    defineMethodObject("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF COLLECTION) (OBJECT OBJECT)) :DOCUMENTATION \"Return true iff `object' is a member of the collection `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Collection::memberP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (REMOVE-DUPLICATES (LIKE SELF)) ((SELF COLLECTION)) :DOCUMENTATION \"Return `self' with duplicates removed.  Preserves the\noriginal order of the remaining members.\" :PUBLIC? TRUE :ABSTRACT? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineFunctionObject("STELLA-COLLECTION?", "(DEFUN (STELLA-COLLECTION? BOOLEAN) ((SELF OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if `self' is a native collection.\")", ((cpp_function_code)(&stellaCollectionP)), NULL);
    defineMethodObject("(DEFMETHOD (NO-DUPLICATES? BOOLEAN) ((SELF COLLECTION)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the collection `self' forbids duplicate values.\")", ((cpp_method_code)(&Collection::noDuplicatesP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NO-DUPLICATES? BOOLEAN) ((SELF SET)) :PUBLIC? TRUE)", ((cpp_method_code)(&Set::noDuplicatesP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (ORDERED? BOOLEAN) ((SELF COLLECTION)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the collection `self' is ordered.\")", ((cpp_method_code)(&Collection::orderedP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (ORDERED? BOOLEAN) ((SELF SEQUENCE)) :PUBLIC? TRUE)", ((cpp_method_code)(&Sequence::orderedP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the set `self' has no members.\")", ((cpp_method_code)(&Set::emptyP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the set `self' has at least one member.\")", ((cpp_method_code)(&Set::nonEmptyP)), ((cpp_method_code)(NULL)));
    defineFunctionObject("TERMINATE-SET?", "(DEFUN (TERMINATE-SET? BOOLEAN) ((SELF SET)))", ((cpp_function_code)(&terminateSetP)), NULL);
    defineMethodObject("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF SET) (OBJECT (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Return `true' iff `object' is a member of the set `self'.\nUses an `eql?' test.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Set::memberP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LENGTH INTEGER) ((SELF SET)) :PUBLIC? TRUE)", ((cpp_method_code)(&Set::length)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT ((SELF SET) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Add `value' to the set `self'.  First checks for duplicate.\")", ((cpp_method_code)(&Set::insert)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (REMOVE SET) ((SELF SET) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Remove all entries in `self' that match `value'.\")", ((cpp_method_code)(&Set::remove)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (REVERSE SET) ((SELF SET)) :PUBLIC? TRUE)", ((cpp_method_code)(&Set::reverse)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (COPY (SET OF (LIKE (ANY-VALUE SELF)))) ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Return a copy of the set `self'.  The conses in the copy are\nfreshly allocated.\")", ((cpp_method_code)(&Set::copy)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD CLEAR ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Make `self' an empty set.\")", ((cpp_method_code)(&Set::clear)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (ALLOCATE-ITERATOR (SET-ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF SET)))", ((cpp_method_code)(&Set::allocateIterator)), ((cpp_method_code)(NULL)));
    defineFunctionObject("TERMINATE-SET-ITERATOR?", "(DEFUN (TERMINATE-SET-ITERATOR? BOOLEAN) ((SELF SET-ITERATOR)))", ((cpp_function_code)(&terminateSetIteratorP)), NULL);
    defineMethodObject("(DEFMETHOD (NEXT? BOOLEAN) ((SELF SET-ITERATOR)))", ((cpp_method_code)(&SetIterator::nextP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (VALUE-SETTER OBJECT) ((SELF SET-ITERATOR) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&SetIterator::valueSetter)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD PRINT-VECTOR ((SELF VECTOR) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_method_code)(&Vector::printVector)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF VECTOR)) :DOCUMENTATION \"Return `true' if `self' has length 0.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (EQL? (ARRAY-SIZE SELF) 0)))", ((cpp_method_code)(&Vector::emptyP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF VECTOR)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if `self' has length > 0.\" :GLOBALLY-INLINE? TRUE (RETURN (> (ARRAY-SIZE SELF) 0)))", ((cpp_method_code)(&Vector::nonEmptyP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (FIRST (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 0)))", ((cpp_method_code)(&Vector::first)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (SECOND (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 1)))", ((cpp_method_code)(&Vector::second)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (THIRD (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 2)))", ((cpp_method_code)(&Vector::third)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (FOURTH (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 3)))", ((cpp_method_code)(&Vector::fourth)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (FIFTH (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 4)))", ((cpp_method_code)(&Vector::fifth)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NTH (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (POSITION INTEGER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH (THE-ARRAY SELF) POSITION)))", ((cpp_method_code)(&Vector::nth)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LAST (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :DOCUMENTATION \"Return the last item in the vector `self'.\")", ((cpp_method_code)(&Vector::last)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (FIRST-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 0) VALUE)))", ((cpp_method_code)(&Vector::firstSetter)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (SECOND-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 1) VALUE)))", ((cpp_method_code)(&Vector::secondSetter)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (THIRD-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 2) VALUE)))", ((cpp_method_code)(&Vector::thirdSetter)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (FOURTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 3) VALUE)))", ((cpp_method_code)(&Vector::fourthSetter)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (FIFTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 4) VALUE)))", ((cpp_method_code)(&Vector::fifthSetter)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF))) (POSITION INTEGER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH (THE-ARRAY SELF) POSITION) VALUE)))", ((cpp_method_code)(&Vector::nthSetter)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LAST-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::lastSetter)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LENGTH INTEGER) ((SELF VECTOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::length)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF VECTOR) (OBJECT OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::memberP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (POSITION INTEGER) ((SELF VECTOR) (OBJECT OBJECT) (START INTEGER)) :DOCUMENTATION \"Return the position of `object' within the vector\n`self' (counting from zero); or return `null' if `object' does not occur within \n`self' (uses an `eql?' test).  If `start' was supplied as non-`null', only \nconsider the portion starting at `start', however, the returned position \nwill always be relative to the entire vector.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::position)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT-AT ((SELF VECTOR) (OFFSET INTEGER) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&Vector::insertAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (COPY (VECTOR OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)) :DOCUMENTATION \"Return a copy of the vector `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::copy)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD CLEAR ((SELF VECTOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::clear)), ((cpp_method_code)(NULL)));
    defineFunctionObject("VECTOR", "(DEFUN (VECTOR VECTOR) (|&REST| (VALUES OBJECT)) :DOCUMENTATION \"Return a vector containing `values', in order.\" :PUBLIC? TRUE)", ((cpp_function_code)(&vector_)), NULL);
    defineMethodObject("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X VECTOR) (Y OBJECT)) :DOCUMENTATION \"Return TRUE iff the vectors `x' and `y' are structurally\nequivalent.  Uses `equal?' to test equality of elements.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::objectEqualP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT-AT ((SELF EXTENSIBLE-VECTOR) (OFFSET INTEGER) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&ExtensibleVector::insertAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT-AT ((SELF EXTENSIBLE-SYMBOL-ARRAY) (OFFSET INTEGER) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&ExtensibleSymbolArray::insertAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT ((SELF VECTOR-SEQUENCE) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Append `value' to the END of the sequence `self'.  \nResize the array if necessary.\" :PUBLIC? TRUE)", ((cpp_method_code)(&VectorSequence::insert)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (REMOVE VECTOR-SEQUENCE) ((SELF VECTOR-SEQUENCE) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Remove `value' from the sequence `self', and left shift\nthe values after it to close the gap.\" :PUBLIC? TRUE)", ((cpp_method_code)(&VectorSequence::remove)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LENGTH INTEGER) ((SELF VECTOR-SEQUENCE)) :PUBLIC? TRUE)", ((cpp_method_code)(&VectorSequence::length)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (ALLOCATE-ITERATOR (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)))", ((cpp_method_code)(&Vector::allocateIterator)), ((cpp_method_code)(NULL)));
    defineFunctionObject("VECTOR-NEXT?", "(DEFUN (VECTOR-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&vectorNextP)), NULL);
    defineMethodObject("(DEFMETHOD (BUT-LAST (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)) :DOCUMENTATION \"Generate all but the last element of the vector `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::butLast)), ((cpp_method_code)(NULL)));
    defineFunctionObject("INITIALIZE-STELLA-HASH-TABLE", "(DEFUN INITIALIZE-STELLA-HASH-TABLE ((SELF STELLA-HASH-TABLE)))", ((cpp_function_code)(&initializeStellaHashTable)), NULL);
    defineFunctionObject("MAKE-STELLA-HASH-TABLE", "(DEFUN (MAKE-STELLA-HASH-TABLE STELLA-HASH-TABLE) ((INITIALSIZE INTEGER)))", ((cpp_function_code)(&makeStellaHashTable)), NULL);
    defineFunctionObject("REHASH-STELLA-HASH-TABLE", "(DEFUN REHASH-STELLA-HASH-TABLE ((SELF STELLA-HASH-TABLE) (NEWSIZE INTEGER)))", ((cpp_function_code)(&rehashStellaHashTable)), NULL);
    defineFunctionObject("STELLA-HASH-TABLE-INSERT-AT", "(DEFUN STELLA-HASH-TABLE-INSERT-AT ((SELF STELLA-HASH-TABLE) (KEY OBJECT) (VALUE OBJECT)))", ((cpp_function_code)(&stellaHashTableInsertAt)), NULL);
    defineFunctionObject("STELLA-HASH-TABLE-REMOVE-AT", "(DEFUN STELLA-HASH-TABLE-REMOVE-AT ((SELF STELLA-HASH-TABLE) (KEY OBJECT)))", ((cpp_function_code)(&stellaHashTableRemoveAt)), NULL);
    defineFunctionObject("STELLA-HASH-TABLE-LOOKUP", "(DEFUN (STELLA-HASH-TABLE-LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF STELLA-HASH-TABLE) (KEY OBJECT)))", ((cpp_function_code)(&stellaHashTableLookup)), NULL);
  }
}

void startupCollections() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (currentStartupTimePhaseP(2)) {
      helpStartupCollections1();
    }
    if (currentStartupTimePhaseP(5)) {
      { Class* clasS = defineClassFromStringifiedSource("STELLA-HASH-TABLE", "(DEFCLASS STELLA-HASH-TABLE (ABSTRACT-HASH-TABLE) :PARAMETERS ((ANY-KEY :TYPE OBJECT) (ANY-VALUE :TYPE OBJECT)) :SLOTS ((THE-TABLE :TYPE (ARRAY () OF KV-CONS)) (SIZE :TYPE INTEGER) (INITIAL-SIZE :TYPE INTEGER :INITIALLY 50) (FREE-ELEMENTS :TYPE INTEGER)))");

        clasS->classConstructorCode = ((cpp_function_code)(&newStellaHashTable));
        clasS->classSlotAccessorCode = ((cpp_function_code)(&accessStellaHashTableSlotValue));
      }
    }
    if (currentStartupTimePhaseP(6)) {
      finalizeClasses();
    }
    if (currentStartupTimePhaseP(7)) {
      helpStartupCollections2();
      defineFunctionObject("STELLA-STRING-HASH-TABLE-LOOKUP", "(DEFUN (STELLA-STRING-HASH-TABLE-LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF STELLA-HASH-TABLE) (KEY STRING)))", ((cpp_function_code)(&stellaStringHashTableLookup)), NULL);
      defineFunctionObject("STARTUP-COLLECTIONS", "(DEFUN STARTUP-COLLECTIONS () :PUBLIC? TRUE)", ((cpp_function_code)(&startupCollections)), NULL);
      { MethodSlot* function = lookupFunction(SYM_COLLECTIONS_STELLA_STARTUP_COLLECTIONS);

        setDynamicSlotValue(function->dynamicSlots, SYM_COLLECTIONS_STELLA_METHOD_STARTUP_CLASSNAME, wrapString("Startup-Collections"), NULL_STRING_WRAPPER);
      }
    }
    if (currentStartupTimePhaseP(8)) {
      finalizeSlots();
      cleanupUnfinalizedClasses();
    }
    if (currentStartupTimePhaseP(9)) {
      registerNativeName(SYM_COLLECTIONS_STELLA_VECTOR, KWD_COLLECTIONS_CPP, KWD_COLLECTIONS_FUNCTION);
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL *USE-STELLA-HASH-TABLES?* BOOLEAN FALSE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL *STELLA-HASH-TABLE-AVG-BUCKET-LENGTH* FLOAT 1.0)");
    }
  }
}

Surrogate* SGT_COLLECTIONS_STELLA_COLLECTION = NULL;

Symbol* SYM_COLLECTIONS_STELLA_VECTOR = NULL;

Keyword* KWD_COLLECTIONS_CPP = NULL;

Keyword* KWD_COLLECTIONS_FUNCTION = NULL;

Surrogate* SGT_COLLECTIONS_STELLA_VECTOR = NULL;

Surrogate* SGT_COLLECTIONS_STELLA_STELLA_HASH_TABLE = NULL;

Symbol* SYM_COLLECTIONS_STELLA_SIZE = NULL;

Symbol* SYM_COLLECTIONS_STELLA_INITIAL_SIZE = NULL;

Symbol* SYM_COLLECTIONS_STELLA_FREE_ELEMENTS = NULL;

Symbol* SYM_COLLECTIONS_STELLA_STARTUP_COLLECTIONS = NULL;

Symbol* SYM_COLLECTIONS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
